'use strict';

var GatheringMap = {
  focusId: 0,
  points: [],

  initialize: function(locations, focusId) {
    // focus on this marker (optional)
    this.focusId = focusId || 0;

    this.points = [];
    var arrayConstructor = [].constructor;
    var stringConstructor = "string".constructor;

    if (locations === undefined || locations === null) {
      this.points = [];
    } else if (locations.constructor === arrayConstructor) {
      this.points = locations;
    } else if (locations.constructor === stringConstructor) {
      this.points = JSON.parse(locations);
    }
  },

  setMarkerGroup: function(points) {
    if (points) {
      var markers = [];
      for (var i = 0; i < points.length; i++) {
        var marker = this.newMarker(points[i]['latlng'], points[i]['options']);
        markers.push(marker);
      }

      this.markerGroup = new L.featureGroup(markers).addTo(this.map);

      this.markerGroup.eachLayer(function (marker) {
        this.markerVisibility(marker);
      }, this);
    }

    return this.markerGroup;
  },

  setMap: function(config) {
    if (config) {
      this.map = L.map('mapid', config).on({
        load: function(e) {
          this.setMarkerGroup(this.points);
          this.focusMarker(this.focusId);
        },

        click: function(e) {
          if (PlatformMap.onMapClick) PlatformMap.onMapClick(e);
        },

        moveend: function(e) {
          if (PlatformMap.onMapMoveEnd) PlatformMap.onMapMoveEnd(e);
        }
      }, this);
    }

    return this.map;
  },

  resetMap: function() {
    if (this.markerGroup) {
      this.markerGroup.eachLayer(function (layer) {
        this.markerGroup.removeLayer(layer);
      }, this);
    }

    if (this.map) {
      this.map.remove();
    }
  },

  addMapboxAttribution: function() {
    var center = this.map.getCenter();
    var attribution = L.control.attribution({prefix: false});

    attribution.addAttribution(
        '<a href="https://www.mapbox.com/about/maps/" target="_blank">&copy; Mapbox</a> ' +
        '<a href="https://openstreetmap.org/about/" target="_blank">&copy; OpenStreetMap</a> ' +
        '<a class="mapbox-improve-map" href="https://www.mapbox.com/map-feedback/#mapbox.streets/' + center.lat + '/' + center.lng + '/17" target="_blank">Improve this map</a>'
    );

    attribution.addTo(this.map);
  },

  addMapboxLogo: function() {
    var logo = L.control({position: 'bottomleft'});

    logo.onAdd = function() {
      return L.DomUtil.create('div', 'mapbox-logo');
    };

    logo.addTo(this.map);
  },

  addImageTileLayer: function(zoom, lat, lng) {
    this.resetMap();

    var tileLayer = L.tileLayer('https://api.mapbox.com/v4/{map_id}/{z}/{x}/{y}.png?access_token={access_token}', {map_id: 'mapbox.streets', access_token: 'pk.eyJ1IjoiZ2F0aGVyZGlnaXRhbCIsImEiOiJjaWwxY2UzcjgwYWE0dWpseDF0NHN3M3FuIn0.7F71-o7JKXPeOleNezhkBw'});

    var map = this.setMap({
      zoomControl: PlatformMap.zoomControl ? PlatformMap.zoomControl() : true,
      attributionControl: false,
      detectRetina: true,
      layers: [tileLayer],
      zoom: zoom
    });

    // we set the view later so our 'load' and 'click' events can be set before they are fired
    map.setView([lat, lng]);

    this.addMapboxLogo();
    this.addMapboxAttribution();

    if (L.Control.geocoder) {
      L.Control.geocoder({defaultMarkGeocode: false}).on('markgeocode', function (e) {
        var bbox = e.geocode.bbox;
        map.fitBounds(bbox);
      }).addTo(map);
    }
  },

  addImageLayer: function(url, color, margin) {
    this.resetMap();

    var map = this.setMap({
      zoomControl: PlatformMap.zoomControl ? PlatformMap.zoomControl() : true,
      attributionControl: false,
      crs: L.CRS.Simple,
      zoomDelta: 0.5,
      zoomSnap: 0,
      minZoom: 1,
      maxZoom: 4,
      zoom: 2
    });

    // get width and height of image
    var image = new Image();

    image.onload = function() {
      // calculate the edges of the image in coordinate space
      var southWest = map.unproject([0, image.height], map.getMaxZoom() - 1);
      var northEast = map.unproject([image.width, 0], map.getMaxZoom() - 1);
      var bounds = new L.LatLngBounds(southWest, northEast);

      // map background color
      if (color && color.length >= 3) {
        map.getContainer().style.backgroundColor = color;
      }

      // map margin
      if (margin && parseInt(margin) > 0) {
        margin = parseInt(margin);
        southWest.lat = southWest.lat - margin;
        southWest.lng = southWest.lng - margin;
        northEast.lat = northEast.lat + margin;
        northEast.lng = northEast.lng + margin;
      }

      // set map bounds
      var mapBounds = new L.LatLngBounds(southWest, northEast);
      map.setMaxBounds(mapBounds);

      // we set the view later so our 'load' and 'click' events can be set before they are fired
      map.fitBounds(bounds);

      // add the image overlay so that it covers the entire map
      L.imageOverlay(url, bounds).addTo(map);
    };

    image.src = url;
  },

  findLayerById: function(layers, id) {
    var foundLayer = null;

    if (layers && layers.length) {
      layers.eachLayer(function (layer) {
        if (layer.id === id) {
          foundLayer = layer;
        }
      }, this);
    }

    return foundLayer;
  },

  blueIcon: function() {
    if (!this.blueMapIcon) {
      var icon = L.Icon.Default;

      this.blueMapIcon = new icon();
    }

    return this.blueMapIcon;
  },

  redIcon: function() {
    if (!this.redMapIcon) {
      var icon = L.Icon.Default.extend({
        options: {
          iconUrl: "red-marker-icon.png",
          iconRetinaUrl: "red-marker-icon-2x.png"
        }
      });

      this.redMapIcon = new icon();
    }

    return this.redMapIcon;
  },

  starIcon: function() {
    if (!this.starMapIcon) {
      var icon = L.Icon.Default.extend({
        options: {
          iconUrl: "star-marker-icon.png",
          iconRetinaUrl: "star-marker-icon-2x.png"
        }
      });

      this.starMapIcon = new icon();
    }

    return this.starMapIcon;
  },

  markerIcon: function(marker) {
    if (marker.options.icon === "star") {
      return this.starIcon();
    } else if (marker.options.icon === "red") {
      return this.redIcon();
    } else {
      return this.blueIcon();
    }
  },

  showMarker: function(marker) {
    if (!this.map.hasLayer(marker)) {
      this.map.addLayer(marker);
    }
  },

  hideMarker: function(marker) {
    if (this.map.hasLayer(marker)) {
      this.map.removeLayer(marker);
    }
  },

  showMarkers: function() {
    if (this.markerGroup) {
      this.markerGroup.eachLayer(function (marker) {
        this.showMarker(marker);
      }, this);
    }
  },

  hideMarkers: function() {
    if (this.markerGroup) {
      this.markerGroup.eachLayer(function (marker) {
        marker.closeTooltip();
        this.hideMarker(marker);
      }, this);
    }
  },

  focusMarker: function(id) {
    if (id > 0 && this.map) {
      this.markerGroup.eachLayer(function (marker) {
        if (marker.id === id) {
          this.map.once('moveend', function () {
            marker.openTooltip();
          });

          this.showMarker(marker);
          this.map.panTo(marker.getLatLng());
        } else if (this.map.hasLayer(marker)) {
          marker.closeTooltip();
          this.markerVisibility(marker);
        }
      }, this);
    }
  },

  markerVisibility: function(marker) {
    if (marker.options.visible) {
      this.showMarker(marker);
    } else {
      this.hideMarker(marker);
    }
  },

  markerToolTip: function(marker) {
    var markerName = marker.options.name;

    if (PlatformMap.markerToolTip) {
      return PlatformMap.markerToolTip(marker);
    } else if (marker.options.object) {
      var objectName = marker.options.object.name + '<br>';
      var objectImage = marker.options.object.image_url ? '<div style="margin-top: 5px"><img src="' + marker.options.object.image_url + '" height="30px" /></div>' : '';
      var tapToView = '<div class="small-text">(tap to view)</div>';

      return markerName + '<br>' + objectImage + objectName + tapToView;
    } else {
      return markerName;
    }
  },

  addMarker: function(marker) {
    this.markerGroup.addLayer(marker);
  },

  removeMarker: function(id) {
    var marker = this.findLayerById(this.markerGroup, id);

    if (marker) {
      marker.closeTooltip();
      this.markerGroup.removeLayer(marker);
    }
  },

  newMarker: function(latlng, options) {
    var marker = new L.marker(latlng, options);

    marker.id = marker.options.id;
    marker.setIcon(this.markerIcon(marker));
    marker.bindTooltip(this.markerToolTip(marker), { interactive: true, opacity: 1.0 }).off('mouseover').off('mouseout');
    marker.on({
      click: function(e) {
        this.map.panTo(e.target.getLatLng());

        // received a click, if the source was not a marker it must be a tooltip
        if (e.originalEvent.srcElement.classList[0] !== "leaflet-marker-icon" && PlatformMap.onToolTipClick) {
          PlatformMap.onToolTipClick(e);
        }
      },

      dragend: function(e) {
        if (PlatformMap.onMarkerDragEnd) PlatformMap.onMarkerDragEnd(e);
      },

      tooltipopen: function(e) {
        if (PlatformMap.onToolTipOpen) PlatformMap.onToolTipOpen(e);
      },

      tooltipclose: function(e) {
        if (PlatformMap.onToolTipClose) PlatformMap.onToolTipClose(e);
      }
    }, this);

    return marker;
  }
};
